/* version v0.1 dd 17 august 2016
 Program for displaying AZ and EL on LCD (16 x 2)screen using Arduino
 Input potentiometers for azimuth and elevation

 Program enables azimuth or elevation display only or both (what_to_display)
 Display resolution for azimuth and elevation can be choosen seperatly 
 The direction of azimuth and display can be choosen
 The azimuth and elevation gear factor have to be completed in the program
 For (offset) calibration the offsets buttons will be used

 Fine tuning of some parameters can be useful e.g. beta and refresh
 
 connections of LCD display and offsetbuttons: see program
 comments are welcome. ON4CDU (at) uba.be
*/

#include <LiquidCrystal.h> // include the library code:
#include <EEPROM.h>

int what_to_display = 3; // CHANGE FOR DISPLAY AZ ONLY TO 1; CHANGE FOR DISPLAY EL ONLY TO 2; CHANGE FOR DISPLAY AZ & EL TO 3
int beta = 7; // FILTER ADJUSTMENT
int refresh = 20; // DISPLAY UPDATE RATE
int az_sensorPin = A0;   // select the input pin for the AZ potentiometer
int el_sensorPin = A1;   // select the input pin for the EL potentiometer
int ledPin = 13;   // select the pin for the LED
int az_sensorValue = 0;  // variable to store the value coming from the sensor
int el_sensorValue = 0;  // variable to store the value coming from the sensor
unsigned long az_temp, el_temp; 

const int offazup = 3;  // pin for button to change az offset
const int offazdn = 4;  // pin for button to change az offset
const int offelup = 5;  // pin for button to change el offset
const int offeldn = 6;  // pin for button to change el offset

// button variables
int stateoffazup = 1;  // buttonstate az offset
int stateoffazdn = 1;  // buttonstate
int stateoffelup = 1;  // buttonstate el offset
int stateoffeldn = 1;  // buttonstate


//display resolution az; MAKE YOUR OWN CHOICE
//int azr = 1; // display resolution 1 degree
int azr = 2; // display resolution .5 degree
//int azr = 5; // display resolution .2 degree
//int azr = 10; // display resolution .1 degree

//display resolution el; MAKE YOUR OWN CHOICE
int elr = 1; // display resolution 1 degree
//int elr = 2; // display resolution .5 degree
//int elr = 5; // display resolution .2 degree
//int elr = 10; // display resolution .1 degree

// calculation and display variables
int el = 0; // elevation value in decimal format
float elg = 0; // el in degrees
int elgi = 0; // el in integer format (10 times el)
float el_gearfactor = 1; // CHANGE THIS: correction for elevation gear (e.g. 1.53)

int az = 0; // azimuth value in decimal format
float azg = 0; // az in degrees
int azgi= 0; // az in integer format
float az_gearfactor = 1; // CHANGE THIS: correction for azimuth gear (e.g. 1.764)

int offsetaz = 0; // offset for azimuth
int offsetel = 0; // offset for elevation

// EEPROM addresses
#define EEPROM_ADDR_STATUS 0
#define EEPROM_ADDR_OFFSET_AZ 1
#define EEPROM_ADDR_OFFSET_EL 3

// general variables needed in program
int d = 0; // timer delay
unsigned int k = 0;

LiquidCrystal lcd(12, 11, 10, 9, 8, 7);// initialize the library with the numbers of the interface pins

void setup()
{
  lcd.begin(16, 2); // set up the LCD's number of columns and rows
  Serial.begin(9600);  // opens serial port, sets data rate to 9600 bps
  pinMode(ledPin, OUTPUT); // output for led
  
  pinMode(offazup, INPUT_PULLUP); // set pin to input and enable pullup resistor
  pinMode(offazdn, INPUT_PULLUP);
  pinMode(offelup, INPUT_PULLUP);
  pinMode(offeldn, INPUT_PULLUP);

  if (1 == EEPROM.read (EEPROM_ADDR_STATUS))
  {
    char* p_mem = (char*)(&offsetaz);
    p_mem [0] = EEPROM.read (EEPROM_ADDR_OFFSET_AZ);
    p_mem [1] = EEPROM.read (EEPROM_ADDR_OFFSET_AZ + 1);
    p_mem = (char*)(&offsetel);
    p_mem [0] = EEPROM.read (EEPROM_ADDR_OFFSET_EL);
    p_mem [1] = EEPROM.read (EEPROM_ADDR_OFFSET_EL + 1);
  }
}

// Main program
void loop()
{
  readstatebuttons();// changing offset?
  d = 300;
  while (stateoffazup == LOW || stateoffazdn == LOW || stateoffelup == LOW || stateoffeldn == LOW) {
    offsetcorrection();
  }
  if (what_to_display == 1 || what_to_display == 3){
    for (int j=0; j<50; j++){
      // read the value from the sensor:
      az_sensorValue = analogRead(az_sensorPin);
      // simple lowpass filter
      az_temp = az_temp - (az_temp>>beta) + (az_sensorValue * 4) ;
      az_sensorValue = (az_temp>>beta);
  
        if ((k%refresh) == 1){
          digitalWrite (ledPin, HIGH); // led on
          transformaz();
          displayaz();
          digitalWrite (ledPin, LOW); // led off
      }
    }
  }

  if (what_to_display == 2 || what_to_display == 3){
    for (int j=0; j<50; j++){
      // read the value from the sensor:
      el_sensorValue = analogRead(el_sensorPin);
      // simple lowpass filter
      el_temp = el_temp - (el_temp>>beta) + (el_sensorValue * 4) ;
      el_sensorValue = (el_temp>>beta);
  
        if ((k%refresh) == 1){
          digitalWrite (ledPin, HIGH); // led on
          transformel();
          displayel();
          digitalWrite (ledPin, LOW); // led off
      }
    }
  }
  k++;
}

void transformaz()  // transform azimuth info to degrees
{
//az_sensorValue = 4092 - az_sensorValue; // for reverse scaling
  azg =(float(az_sensorValue) * 360 * az_gearfactor)/4093; // transformation to degrees
  azg = azr* azg;
  azg = round (azg);
  azg = azg/azr;
  azg = 10 * azg;
  azgi = azg;
}

void transformel()  // transform elevation info to degrees
{
//el_sensorValue = 4092 - el_sensorValue; // for reverse scaling
  elg = (float(el_sensorValue) * 360 * el_gearfactor)/4093;
  
  elg = elr * elg;
  elg = round (elg);
  elg = elg/elr;
  elg = 10 * elg;
  elgi = elg;
  
  }

void displayaz() // display azimuth value
{
  int azgd; // az value for display purposes
  char buffer [16]; //buffer for dislay a line on LCD
  
  azgd = azgi + (offsetaz)/azr;
 //Serial.println (offsetaz); // for test purposes
  if (azgd >= 3600) {
    azgd = azgd - 3600;
  }
  else if (azgd < 0) {
    azgd = 3600 + azgd; // az has always a positive value
  }
 
  int upper_part = azgd/10;
  int lower_part = azgd%10; 
  //Serial.println(upper_part); // for test purposes
  //Serial.println(lower_part); // for test purposes
 
  if (azr <=1) {
    sprintf (buffer, "Azimuth   %3d ", upper_part);
  }
  else {
    sprintf (buffer, "Azimuth   %3d.%1d ", upper_part, lower_part);
  }
  
  lcd.setCursor (0, 0);
  lcd.print (buffer);
}

void displayel()  // display elevation value
{
  int elgd; // el value for display purposes
  int neg = 0; // boolean for display negative el values
  char buffer [16]; //buffer for dislay a line on LCD
   
  elgd = elgi + offsetel/elr; // display value corrected with offset
 
if (elgd >= 3600) {
    elgd = elgd - 3600;
  }
  else if (elgd < 0) {
    elgd = 3600 + elgd; // el has now always a positive value
  }
   
if (elgd > 2700) {
    elgd = 3600 - elgd;
    neg = 1;
  }

  int upper_part = elgd/10;
  int lower_part = elgd % 10;
  //Serial.println(upper_part); // for test purposes
  //Serial.println(lower_part); // for test purposes
   
 if (neg==1) {
  if (elr <=1){
    sprintf (buffer, "Elevation -%2d", upper_part);
  }
  else {
    sprintf (buffer, "Elevation -%2d.%1d ", upper_part, lower_part);
  }
    neg = 0;
 }
 else {
  if (elr <=1){
   sprintf (buffer, "Elevation %3d", upper_part); 
  }
  else {
    sprintf (buffer, "Elevation %3d.%1d ", upper_part, lower_part);
  }
 }

  lcd.setCursor (0, 1);
  lcd.print (buffer);
}

void readstatebuttons()
{
  stateoffazup = digitalRead(offazup);
  stateoffazdn = digitalRead(offazdn);
  stateoffelup = digitalRead(offelup);
  stateoffeldn = digitalRead(offeldn);
}

void offsetcorrection()
{
  if (stateoffazup == LOW)
  {
    offsetaz = offsetaz + 10;
    displayaz();
    delay (d);
    if (d > 30) {
      d = d - 30;
    }
  }
  if (stateoffazdn == LOW)
  {
    offsetaz = offsetaz - 10;
    displayaz();
    delay (d);
    if (d > 30) {
      d = d - 30;
    }
  }
  if (stateoffelup == LOW)
  {
    offsetel = offsetel + 10;
    displayel();
    delay (d);
    if (d > 30) {
      d = d - 30;
    }
  }
  if (stateoffeldn == LOW)
  {
    offsetel = offsetel - 10;
    displayel();
    delay (d);
    if (d > 30) {
      d = d - 30;
    }
  }
  
//offsetaz = 0; //useful for filling offset zeros in EEPROM
//offsetel = 0; //useful for filling offset zeros in EEPROM

  // writing offset data to EEPROM
  EEPROM.write (EEPROM_ADDR_STATUS, 1);
  char* p_mem = (char*)(&offsetaz);
  EEPROM.write (EEPROM_ADDR_OFFSET_AZ  , p_mem [0]);
  EEPROM.write (EEPROM_ADDR_OFFSET_AZ + 1, p_mem [1]);
  p_mem = (char*)(&offsetel);
  EEPROM.write (EEPROM_ADDR_OFFSET_EL  , p_mem [0]);
  EEPROM.write (EEPROM_ADDR_OFFSET_EL + 1, p_mem [1]);

  readstatebuttons();
}
