/* version v0.1 dd 24 august 2016. Software for sensorboard HW_sensorboard_analog
 for reading one or two potentiometers for reading azimuth and elevation.
 
 Input potentiometers for azimuth and elevation

output from this program is a frame of 6 bytes: del azhi azlo elhi ello 13(hex);
  del is:
  -- D0 for azimuth data only
  -- D1 for elevation data only
  -- D2 for azimuth and elevation data
  
Azimuth and elevation data is split in into 2 bytes for tansport (high byte and low byte)
Configure which data to measure at: what_to_measure (at the start of the program)
Fine tuning of some parameters can be useful e.g. beta and refresh
 
comments are welcome. ON4CDU (at) uba.be
*/

int what_to_measure = 0; // 0 for az only; 1 for el only; 2 for az & el
int beta = 7; // FILTER ADJUSTMENT
int refresh = 20; // DISPLAY UPDATE RATE
int az_sensorPin = A0;   // select the input pin for the AZ potentiometer
int el_sensorPin = A1;   // select the input pin for the EL potentiometer
int ledPin = 13;   // select the pin for the LED
int az_sensorValue = 0;  // variable to store the value coming from the sensor
int el_sensorValue = 0;  // variable to store the value coming from the sensor
unsigned long az_temp, el_temp;
 
int mask_lo_byte = 0xFF; // 000011111111; mask for getting least significant byte
int rxtx_control = 3;

unsigned char del = 0;      // delimiter frame; 
unsigned char AZ_hi_byte = 0; // azimuth high byte value
unsigned char AZ_lo_byte = 0; // azimuth low byte value
unsigned char EL_hi_byte = 0; // elevation high byte value
unsigned char EL_lo_byte = 0; // elevation low byte value
unsigned char endframe = 0x13;  // end of frame byte

// general variables needed in program
unsigned int k = 0;

void decode_az_data()
{
  AZ_hi_byte = (az_sensorValue) >> 8; // shift 8 bits to form az high byte
  AZ_lo_byte = az_sensorValue & mask_lo_byte;
}

void decode_el_data()
{
  EL_hi_byte = (el_sensorValue) >> 8; // shift 8 bits to form az high byte
  EL_lo_byte = el_sensorValue & mask_lo_byte;
}

void sendframe()
{
  int i;
  char buf [6] = {del, EL_hi_byte, EL_lo_byte, AZ_hi_byte, AZ_lo_byte, endframe};

  for (i = 0; i < 6; i++)
  {
    digitalWrite (13, HIGH); // led on
    digitalWrite (rxtx_control, HIGH);
    Serial.write(buf[i]);
    //Serial.println (buf[i]);
    delay(2);
    digitalWrite (rxtx_control, LOW);
    digitalWrite (13, LOW); // led off
  }
}

void setup()
{
  Serial.begin(9600);  // opens serial port, sets data rate to 9600 bps
  pinMode(ledPin, OUTPUT); // output for led
  pinMode(rxtx_control, OUTPUT); // direction control rs485 
}

// Main program
void loop()
{
  if (what_to_measure == 0 || what_to_measure == 2){
    del = 0xD0;
    for (int j=0; j<50; j++){
      // read the value from the sensor:
      az_sensorValue = analogRead(az_sensorPin);
      // simple lowpass filter
      az_temp = az_temp - (az_temp>>beta) + (az_sensorValue * 4) ;
      az_sensorValue = (az_temp>>beta);
    }
  }
   
  if (what_to_measure == 1 || what_to_measure == 2){
    del = 0xD1;
    for (int j=0; j<50; j++){
      // read the value from the sensor:
      el_sensorValue = analogRead(el_sensorPin);
      // simple lowpass filter
      el_temp = el_temp - (el_temp>>beta) + (el_sensorValue * 4) ;
      el_sensorValue = (el_temp>>beta);
    }
  }
    if ((k%refresh) == 1){
      decode_az_data();
      decode_el_data();
      if (what_to_measure == 2) {
        del = 0xD2;
      }   
      sendframe();
    }
  k++;
}




 

